package zap

import (
	"git.perx.ru/perxis/perxis-go/logs"
	"go.uber.org/zap/zapcore"
)

// WriteSyncer отвечает за хранение и синхронизацию logs.Entry
type WriteSyncer interface {
	Write(entry *logs.Entry) error
	Sync() error
}

// Core кодирует zapcore.Entry в logs.Entry и отправляет их в WriteSyncer
type Core struct {
	zapcore.LevelEnabler

	ws  WriteSyncer
	enc Encoder
}

func NewCore(writeSyncer WriteSyncer) *Core {
	return &Core{
		LevelEnabler: zapcore.InfoLevel,
		ws:           writeSyncer,
		enc:          NewEntryEncoder(),
	}
}

func (core *Core) With(fields []zapcore.Field) zapcore.Core {
	enc := core.enc.Clone()
	enc.AddFields(fields)
	return &Core{
		LevelEnabler: core.LevelEnabler,
		ws:           core.ws,
		enc:          enc,
	}
}

func (core *Core) Check(entry zapcore.Entry, checkedEntry *zapcore.CheckedEntry) *zapcore.CheckedEntry {
	if core.Enabled(entry.Level) {
		return checkedEntry.AddCore(entry, core)
	}
	return checkedEntry
}

func (core *Core) Write(entry zapcore.Entry, fields []zapcore.Field) error {
	encodedEntry, err := core.enc.EncodeEntry(entry, fields)
	if err != nil {
		return err
	}
	return core.ws.Write(encodedEntry)
}

func (core *Core) Sync() error {
	return core.ws.Sync()
}
